package com.example.api.core

import com.example.api.core.data.core.ApiService
import com.example.api.core.data.dto.RegisterUserDto
import com.example.api.core.data.dto.SignInUserDto
import com.jakewharton.retrofit2.converter.kotlinx.serialization.asConverterFactory
import junit.framework.TestCase.assertEquals
import kotlinx.coroutines.runBlocking
import kotlinx.serialization.json.Json
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import okhttp3.mockwebserver.MockResponse
import okhttp3.mockwebserver.MockWebServer
import org.junit.After
import org.junit.Before
import org.junit.Test
import retrofit2.Retrofit
import retrofit2.create

/**
 * Автор: Манякин Дмитрий (user5)
 * Дата создания: 27.05.2025
 * */

class ServiceTest {

    private lateinit var mockWebServer: MockWebServer
    private lateinit var service: ApiService

    @Before
    fun setup() {
        mockWebServer = MockWebServer()
        mockWebServer.start()

        val json = Json { ignoreUnknownKeys = true }

        service = Retrofit.Builder()
            .baseUrl(mockWebServer.url("/"))
            .addConverterFactory(json.asConverterFactory("application/json".toMediaType()))
            .client(OkHttpClient.Builder().addInterceptor(HttpLoggingInterceptor().apply {
                level = HttpLoggingInterceptor.Level.BODY
            }).build())
            .build().create<ApiService>()
    }

    @Test
    fun test_registration() = runBlocking {
        val response = """
            {
              "collectionId": "_pb_users_auth_",
              "collectionName": "users",
              "created": "2025-05-27 06:08:13.833Z",
              "dateBirthday": "",
              "emailVisibility": false,
              "firstname": "",
              "gender": "",
              "id": "w20q0pj1e2824oz",
              "lastname": "",
              "secondname": "",
              "updated": "2025-05-27 06:08:13.833Z",
              "verified": false
            }
        """.trimIndent()

        mockWebServer.enqueue(MockResponse().setResponseCode(200).setBody(response))

        val responseResult = service.register(
            RegisterUserDto(
                email = "email@test.com",
                password = "12345678",
                passwordConfirm = "12345678"
            )
        )

        val request = mockWebServer.takeRequest()

        assertEquals("w20q0pj1e2824oz", responseResult.id)
        assertEquals("2025-05-27 06:08:13.833Z", responseResult.created)

        assertEquals("POST", request.method)
        assertEquals("/collections/users/records", request.path)
    }

    @Test
    fun test_sign_in() = runBlocking {
        val response = """
            {
              "record": {
                "collectionId": "_pb_users_auth_",
                "collectionName": "users",
                "created": "2025-05-27 06:08:13.833Z",
                "dateBirthday": "",
                "email": "example123@test.ru",
                "emailVisibility": false,
                "firstname": "",
                "gender": "",
                "id": "w20q0pj1e2824oz",
                "lastname": "",
                "secondname": "",
                "updated": "2025-05-27 06:08:13.833Z",
                "verified": false
              },
              "token": "eyJhbGciOiJIUzIqwwe1NiIsInR5cCI6IkpXVCJg9.eyJjb2xsZWN0aW9uSWheQiOiJfqcGggJfdXNlcnNfYXV0aFasd8iLCJleHAiOjE3NDg5MzA5MjAsImlkIjoidzIwcTBwajFlMjgyNG96IiwicmVmcmVzaGFibGUiOnRydWUsInR5cGUiOiJhdXRoIn0.7Sc_NB_CgnwEfXWp0NjsnDT35gGVXzFJwwLUVD730Gw"
            }
        """.trimIndent()

        mockWebServer.enqueue(MockResponse().setResponseCode(200).setBody(response))

        val responseResult = service.auth(
            SignInUserDto(
                identity = "example123@test.ru",
                password = "12345678"
            )
        )

        val request = mockWebServer.takeRequest()

        assertEquals("example123@test.ru", responseResult.record.email)
        assertEquals(
            "eyJhbGciOiJIUzIqwwe1NiIsInR5cCI6IkpXVCJg9.eyJjb2xsZWN0aW9uSWheQiOiJfqcGggJfdXNlcnNfYXV0aFasd8iLCJleHAiOjE3NDg5MzA5MjAsImlkIjoidzIwcTBwajFlMjgyNG96IiwicmVmcmVzaGFibGUiOnRydWUsInR5cGUiOiJhdXRoIn0.7Sc_NB_CgnwEfXWp0NjsnDT35gGVXzFJwwLUVD730Gw",
            responseResult.token
        )

        assertEquals("POST", request.method)
        assertEquals("/collections/users/auth-with-password", request.path)
    }

    @After
    fun teardown() {
        mockWebServer.shutdown()
    }
}